@@ -1,6 +1,6 @@
-REM  $Header: /Users/claude/fuzz/lib/Genezzo/RCS/dict.sql,v 1.13 2007/01/09 09:23:28 claude Exp claude $
+REM  $Header: /Users/claude/fuzz/lib/Genezzo/RCS/dict.sql,v 1.14 2007/07/04 07:40:40 claude Exp claude $
 REM
-REM $Revision: 1.13 $
+REM $Revision: 1.14 $
 REM
 REM copyright (c) 2005, 2006 Jeffrey I Cohen, all rights reserved, worldwide
 REM
@@ -37,5 +37,8 @@ REM load sql scalar functions and comparison functions
 select HavokUse('Genezzo::Havok::SQLScalar') from dual;
 select HavokUse('Genezzo::Havok::SQLCompare') from dual;
 
+REM load the space management hooks
+REM select HavokUse('Genezzo::SpaceMan::SMHook') from dual;
+
 REM always commit changes!!
 commit ;
